### R code from vignette source 'lecture3_2015.Rnw'
### Encoding: UTF-8

###################################################
### code chunk number 1: loadnations
###################################################
u <- url("http://www.quantoid.net/files/essex/nations.rda")
load(u)
close(u)
nations[1:5, 1:5]


###################################################
### code chunk number 2: dissim
###################################################
d <- (9-nations)^2


###################################################
### code chunk number 3: double_center_2
###################################################
doubleCenter <- function(x){
  p <- dim(x)[1]
  n <- dim(x)[2]
  -(x-matrix(apply(x,1,mean),nrow=p,ncol=n) -
     t(matrix(apply(x,2,mean),nrow=n,ncol=p)) + mean(x))/2
}
D <- doubleCenter(d)
ev <- eigen(D)


###################################################
### code chunk number 4: make_coords_torgerson
###################################################
makeCoords <- function(obj){
    scaleFac <- sqrt(max((abs(obj$vec[,1]))^2 + (abs(obj$vec[,2]))^2))
    x <- obj$vec[,1]*(1/T)*sqrt(obj$val[1])
    y <- obj$vec[,2]*(1/T)*sqrt(obj$val[2])
    coords <- cbind(x,y)
    return(coords)
}
coords <- makeCoords(ev)
rownames(coords) <- rownames(D)


###################################################
### code chunk number 5: torg_plot
###################################################
coords[,2] <- -1*coords[,2]
plot(coords[,1], coords[,2], pch=16, col="gray", font=2,
       main="Torgerson Solution", xlab="", ylab="",
       xlim=c(-6,6), ylim=c(-6,6), asp=1)
text(coords[,1], coords[,2], rownames(nations),
    pos=c(4,4,2,4,4,4,4,4,4,4,1,4), offset=0.35, col="black", font=2)
abline(a=0, b=1, lwd=2, lty=2)
abline(a=0, b=-1, lwd=2, lty=2)


###################################################
### code chunk number 6: smacof
###################################################
library(smacof)
smacof_metric_result <- smacofSym(delta=d, ndim=2, weightmat=NULL,
    type="ratio", itmax = 1000, eps=0.000001)
conf <- smacof_metric_result$conf


###################################################
### code chunk number 7: smacof_metric_nations
###################################################
plot(conf[,1],conf[,2],
    main="SMACOF (Metric) Solution", xlab="", ylab="",
    xlim=c(-1,1), ylim=c(-1,1), asp=1, type="n")
points(conf[,1], conf[,2], pch=16, col="gray")
text(-0.9, 0.9, paste("Stress = ",
    round(smacof_metric_result$stress,3)), font=2)
text(conf[,1], conf[,2], rownames(nations),
    pos=rep(4,nrow(nations)), offset=00.25, col="black", font=2)
abline(a=0, b=-3/5, lwd=2, lty=2)
abline(a=0, b=5/3, lwd=2, lty=2)


###################################################
### code chunk number 8: compmds
###################################################
cor(dist(conf), dist(coords))


###################################################
### code chunk number 9: compmdsfig
###################################################
plot(dist(conf), dist(coords))


###################################################
### code chunk number 10: stressplot
###################################################
stressPlot <- function(obj, maxdim=5){
    s <- NULL
    for(i in 1:maxdim){
        s <- c(s, update(obj, ndim=i)$stress)
    }
    plot(1:maxdim, s, type="o", pch=16, 
        xlab = "# Dimensions", ylab = "Stress")
}
stressPlot(smacof_metric_result)


###################################################
### code chunk number 11: nations_miss
###################################################
d <- (9-nations)^2
d[1,2] <- d[2,1] <- NA
weightmat <- !is.na(d)
d[is.na(d)] <- mean(d, na.rm=TRUE)


###################################################
### code chunk number 12: smacofmiss
###################################################
missing_result <- smacofSym(delta=d, ndim=2, weightmat=weightmat,
    type="ratio", itmax = 1000, eps=0.000001)
miss.conf <- missing_result$conf
plot(miss.conf[,1], miss.conf[,2],
    main="SMACOF (Metric) Solution with Missing Value
    for Brazil-Congo Dissimilarity", xlab="", ylab="",
    xlim=c(-1,1), ylim=c(-1,1), asp=1, type="n")
points(miss.conf[,1], miss.conf[,2], pch=16, col="gray")
text(-0.9,0.9,paste("Stress = ",
    round(missing_result$stress,3)),font=2)
text(miss.conf[,1], miss.conf[,2], rownames(nations),
    pos=rep(4,nrow(nations)), offset=00.25, col="black", font=2)
abline(a=0, b=-3/5, lwd=2, lty=2)
abline(a=0, b=5/3, lwd=2, lty=2)


###################################################
### code chunk number 13: smacof_nonmetric_2
###################################################
d <- (9-nations)^2
smacof_nonmetric_result <- smacofSym(delta=d, ndim=2, type="ordinal")
nm.conf <- smacof_nonmetric_result$conf


###################################################
### code chunk number 14: smacof_nonmetric_nations
###################################################
plot(nm.conf[,1], nm.conf[,2], type="n", asp=1, cex.lab=1.4, cex.axis=1.3, cex.main=1.6,
       main="Non-Metric (SMACOF) Solution",
       xlab="",
       ylab="",
       xlim=c(-1.0,1.0),ylim=c(-1.0,1.0),font=2)
points(nm.conf[,1],nm.conf[,2],pch=16,col="gray", cex=1.1)
text(-0.8,0.9,paste("Stress = ",round(smacof_nonmetric_result$stress,3)),font=2, cex=1.2)
text(nm.conf[,1],nm.conf[,2],rownames(nations),
    pos=c(4,4,4,4,4,4,4,4,4,4,4,2),offset=00.25,col="black",font=2, cex=1.1)
abline(a=0,b=-3/5,lwd=2,lty=2)
abline(a=0,b=5/3,lwd=2,lty=2)


###################################################
### code chunk number 15: corrs
###################################################
diag(cor(conf, nm.conf))
cor(dist(conf), dist(nm.conf))


###################################################
### code chunk number 16: rot1
###################################################
cos.deg <- function(x)cos((x*pi)/180)
sin.deg <- function(x)sin((x*pi)/180)
A <- matrix(c(cos.deg(-30), sin.deg(-30), 
    -sin.deg(-30), cos.deg(-30)), nrow=2, ncol=2)
rot.mds <- smacof_metric_result$conf %*% A


###################################################
### code chunk number 17: comprot
###################################################
library(lattice)
plot.dat <- as.data.frame(rbind(conf, rot.mds))
names(plot.dat) <- c("Dim1", "Dim2")
plot.dat$config <- factor(rep(c(1,2), each=nrow(conf)), 
    labels = c("Raw", "Rotated"))
trellis.par.set(superpose.symbol=list(col=c("gray33", "gray66"), pch=c(15,16)))
xyplot(Dim2 ~ Dim1, groups=config, data=plot.dat, 
    auto.key=list(space="top"))


###################################################
### code chunk number 18: rot2
###################################################
library(MCMCpack)
rot.nm <- procrustes(nm.conf, conf, translation=F, dilation=T)
# corrlation between raw n-m and m configurations
diag(cor(nm.conf, conf))

# corrlation between rotated n-m and m configurations
diag(cor(rot.nm$X.new, conf))


###################################################
### code chunk number 19: sizerot
###################################################
acos.deg <- function(x)acos(x)*180/pi
acos.deg(rot.nm$R[1,1])


###################################################
### code chunk number 20: 112
###################################################
library(foreign)
dat <- read.dta("http://www.quantoid.net/files/essex/sen112kh.dta")
votes <- as.matrix(dat[,-c(1:9)])
votes[which(votes %in% c(1:3), arr.ind=T)] <- 1
votes[which(votes %in% c(0, 7:9), arr.ind=T)] <- 9
votes[which(votes %in% c(4:6), arr.ind=T)] <- 0
votes[which(votes == 9, arr.ind=T)] <- NA
D <- dist(votes)^2
w <- D
w[which(!is.na(w))] <- 1
w[which(is.na(w))] <- 0
D[which(is.na(D))] <- .5
mds.agree <- smacofSym(D, ndim=2, weightmat = w)


###################################################
### code chunk number 21: visagree
###################################################
library(car)
pch.party <- recode(dat$party, "100='D'; 200='R'; 328='I'", 
    as.factor.result=F)
agree.conf <- mds.agree$conf
agree.conf[,1] <- -agree.conf[,1]
plot(agree.conf, pch=pch.party)


###################################################
### code chunk number 22: readsc
###################################################
u <- url("http://www.quantoid.net/files/essex/supremecourt.2011.rda")
load(u)
close(u)


###################################################
### code chunk number 23: bsmds
###################################################
library(bsmds)
votes <- votes[-c(24,57), ]
lopsided <- apply(votes, 2, function(x)mean(x == 1, na.rm=T))
outs <- which(lopsided > .95 | lopsided < .05)
votes <- votes[,-outs]
library(bsmds)
out <- bsmds(t(votes), dist.fun = "dist", R=50)


###################################################
### code chunk number 24: plotbsconfig
###################################################
plot(out, id="none", key.side="none")


###################################################
### code chunk number 25: readsc2
###################################################
u <- url("http://www.quantoid.net/files/essex/supreme_court_2010-2013.rda")
load(u)
close(u)


###################################################
### code chunk number 26: fpindscal
###################################################
u <- url("http://www.quantoid.net/files/essex/french.parties.individuals.rda")
load(u)
close(u)
fi <- na.omit(french.parties.individuals)
parties <- lapply(1:50, function(x)dist(t(fi[x,]))^2 + .001)
indscal.result.2dim <- smacofIndDiff(delta=parties, ndim=2,
    constraint="indscal")


###################################################
### code chunk number 27: plot.indscal
###################################################
plot.indscal <- function(obj, pch=NULL, ...){
    conf <- obj$gspace[,1:2]
    yl <- range(conf[,2]) + c(-1,1)*.05*diff(range(conf[,2]))
    xl <- range(conf[,1]) + c(-1,1)*.05*diff(range(conf[,1]))
    require(lattice)
    if(is.null(pch)){
        pch <- 1:nrow(conf)
    }
    conf <- as.data.frame(conf)
    conf <- conf[order(conf[,1]), ]
    conf$stim <- factor(1:nrow(conf), labels=rownames(conf))
    if(is.null(pch)){
       pch <- 1:nrow(conf) 
    }
    trellis.par.set(superpose.symbol=list(col="black", pch=pch))
    p <- xyplot(conf[,2] ~ conf[,1], groups=stim, data=conf, 
        xlim=c(-1.5, 1.5), ylim=c(-1.5, 1.5), 
        xlab = "First Dimension", ylab="Second Dimension",
        auto.key=list(space="top", columns=3))
    return(p)   
}



###################################################
### code chunk number 28: pi
###################################################
plot.indscal(indscal.result.2dim)


###################################################
### code chunk number 29: pw
###################################################
w <- t(sapply(indscal.result.2dim$cweights, diag))
plot(w)


